//
// File: uav_fdm3d_types.h
//
// Code generated for Simulink model 'uav_fdm3d'.
//
// Model version                  : 1.16
// Simulink Coder version         : 9.0 (R2018b) 24-May-2018
// C/C++ source code generated on : Sun Aug 30 11:34:49 2020
//
// Target selection: ert.tlc
// Embedded hardware selection: Intel->x86-64 (Windows64)
// Code generation objectives: Unspecified
// Validation result: Not run
//
#ifndef RTW_HEADER_uav_fdm3d_types_h_
#define RTW_HEADER_uav_fdm3d_types_h_
#include "rtwtypes.h"
#include "builtin_typeid_types.h"

// Parameters (default storage)
typedef struct P_uav_fdm3d_T_ P_uav_fdm3d_T;

// Forward declaration for rtModel
typedef struct tag_RTM_uav_fdm3d_T RT_MODEL_uav_fdm3d_T;

#endif                                 // RTW_HEADER_uav_fdm3d_types_h_

//
// File trailer for generated code.
//
// [EOF]
//
