//
// File: uav_fdm3d_data.cpp
//
// Code generated for Simulink model 'uav_fdm3d'.
//
// Model version                  : 1.16
// Simulink Coder version         : 9.0 (R2018b) 24-May-2018
// C/C++ source code generated on : Sun Aug 30 11:34:49 2020
//
// Target selection: ert.tlc
// Embedded hardware selection: Intel->x86-64 (Windows64)
// Code generation objectives: Unspecified
// Validation result: Not run
//
#include "uav_fdm3d.h"
#include "uav_fdm3d_private.h"

// Invariant block signals (default storage)
const ConstB_uav_fdm3d_T uav_fdm3d_ConstB = {
  0.0
  ,                                    // '<S30>/Unit Conversion'
  0.0
  ,                                    // '<S16>/SinCos'
  1.0
  ,                                    // '<S16>/SinCos'
  0.99664718933525254
  ,                                    // '<S34>/Sum'
  0.99330562000985867
  ,                                    // '<S35>/Product1'
  0.00669437999014133
  ,                                    // '<S35>/Sum1'
  0.08181919084262157
  ,                                    // '<S35>/sqrt'
  0.0066943799901413295
  ,                                    // '<S31>/Product2'
  0.99330562000985867
  // '<S31>/Sum1'
};

//
// File trailer for generated code.
//
// [EOF]
//
