__author__ = 'Zheng GONG'
__email__ = 'matt@matthewgong.com'
__url__ = 'https://sh.matthewgong.com/matt/cluster_uav_node'
__copyright__ = 'Copyright (C) 2020 ' + __author__
__license__ = 'MIT License'
__version__ = '0.1.5'
